searchNodes=[{"ref":"Ersatz.html","title":"Ersatz","type":"module","doc":"Ersatz is a library for defining concurrent mocks in Elixir. The library follows the principles outlined in &quot;Mocks and explicit contracts&quot;, summarized below: No ad-hoc mocks. You can only create mocks based on behaviours No dynamic generation of modules during tests. Mocks are preferably defined in your test_helper.exs or in a setup_all block and not per test Concurrency support. Tests using the same mock can still use async: true Rely on pattern matching and function clauses for asserting on the input instead of complex expectation rules Example As an example, imagine that your library defines a calculator behaviour: defmodule MyApp.Calculator do @callback add(integer(), integer()) :: integer() @callback mult(integer(), integer()) :: integer() end If you want to mock the calculator behaviour during tests, the first step is to define the mock, usually in your test_helper.exs: Ersatz.defmock(MyApp.CalcMock, for: MyApp.Calculator) Now in your tests, you can define expectations and verify them: use ExUnit.Case, async: true import Ersatz # Make sure mocks are verified when the test exits setup :verify_on_exit! test &quot;invokes add and mult&quot; do MyApp.CalcMock |&gt; expect(:add, fn x, y -&gt; x + y end) |&gt; expect(:mult, fn x, y -&gt; x * y end) assert MyApp.CalcMock.add(2, 3) == 5 assert MyApp.CalcMock.mult(2, 3) == 6 end In practice, you will have to pass the mock to the system under the test. If the system under test relies on application configuration, you should also set it before the tests starts to keep the async property. Usually in your config files: config :my_app, :calculator, MyApp.CalcMock Or in your test_helper.exs: Application.put_env(:my_app, :calculator, MyApp.CalcMock) All expectations are defined based on the current process. This means multiple tests using the same mock can still run concurrently unless the Ersatz is set to global mode. See the &quot;Multi-process collaboration&quot; section. Multiple behaviours Ersatz supports defining mocks for multiple behaviours. Suppose your library also defines a scientific calculator behaviour: defmodule MyApp.ScientificCalculator do @callback exponent(integer(), integer()) :: integer() end You can mock both the calculator and scientific calculator behaviour: Ersatz.defmock(MyApp.SciCalcMock, for: [MyApp.Calculator, MyApp.ScientificCalculator]) Compile-time requirements If the mock needs to be available during the project compilation, for instance because you get undefined function warnings, then instead of defining the mock in your test_helper.exs, you should instead define it under test/support/mocks.ex: Ersatz.defmock(MyApp.CalcMock, for: MyApp.Calculator) Then you need to make sure that files in test/support get compiled with the rest of the project. Edit your mix.exs file to add the test/support directory to compilation paths: def project do [ ... elixirc_paths: elixirc_paths(Mix.env), ... ] end defp elixirc_paths(:test), do: [&quot;test/support&quot;, &quot;lib&quot;] defp elixirc_paths(_), do: [&quot;lib&quot;] Multi-process collaboration Ersatz supports multi-process collaboration via two mechanisms: explicit allowances global mode The allowance mechanism can still run tests concurrently while the global one doesn&#39;t. We explore both next. Explicit allowances An allowance permits a child process to use the expectations and stubs defined in the parent process while still being safe for async tests. test &quot;invokes add and mult from a task&quot; do MyApp.CalcMock |&gt; expect(:add, fn x, y -&gt; x + y end) |&gt; expect(:mult, fn x, y -&gt; x * y end) parent_pid = self() Task.async(fn -&gt; MyApp.CalcMock |&gt; allow(parent_pid, self()) assert MyApp.CalcMock.add(2, 3) == 5 assert MyApp.CalcMock.mult(2, 3) == 6 end) |&gt; Task.await end Note: if you&#39;re running on Elixir 1.8.0 or greater and your concurrency comes from a Task then you don&#39;t need to add explicit allowances. Instead $callers is used to determine the process that actually defined the expectations. Global mode Ersatz supports global mode, where any process can consume mocks and stubs defined in your tests. To manually switch to global mode use: set_ersatz_global() which can be done as a setup callback: setup :set_ersatz_global setup :verify_on_exit! test &quot;invokes add and mult from a task&quot; do MyApp.CalcMock |&gt; expect(:add, fn x, y -&gt; x + y end) |&gt; expect(:mult, fn x, y -&gt; x * y end) Task.async(fn -&gt; assert MyApp.CalcMock.add(2, 3) == 5 assert MyApp.CalcMock.mult(2, 3) == 6 end) |&gt; Task.await end The global mode must always be explicitly set per test. By default mocks run on private mode. You can also automatically choose global or private mode depending on if your tests run in async mode or not. In such case Ersatz will use private mode when async: true, global mode otherwise: setup :set_ersatz_from_context"},{"ref":"Ersatz.html#clear_mock_calls/1","title":"Ersatz.clear_mock_calls/1","type":"function","doc":""},{"ref":"Ersatz.html#defmock/2","title":"Ersatz.defmock/2","type":"function","doc":"Defines a mock with the given name :for the given behaviour(s). Ersatz.defmock(MyMock, for: MyBehaviour) With multiple behaviours: Ersatz.defmock(MyMock, for: [MyBehaviour, MyOtherBehaviour]) Skipping optional callbacks By default, functions are created for all callbacks, including all optional callbacks. But if for some reason you want to skip optional callbacks, you can provide the list of callback names to skip (along with their arities) as :skip_optional_callbacks: Ersatz.defmock(MyMock, for: MyBehaviour, skip_optional_callbacks: [on_success: 2]) This will define a new mock (MyMock) that has a defined function for each callback on MyBehaviour except for on_success/2. Note: you can only skip optional callbacks, not required callbacks. You can also pass true to skip all optional callbacks, or false to keep the default of generating functions for all optional callbacks."},{"ref":"Ersatz.html#get_mock_calls/1","title":"Ersatz.get_mock_calls/1","type":"function","doc":""},{"ref":"Ersatz.html#set_ersatz_from_context/1","title":"Ersatz.set_ersatz_from_context/1","type":"function","doc":"Chooses the Ersatz mode based on context. When async: true is used the mode is :private, otherwise :global is chosen. setup :set_ersatz_from_context"},{"ref":"Ersatz.html#set_ersatz_global/1","title":"Ersatz.set_ersatz_global/1","type":"function","doc":"Sets the Ersatz to global mode, where mocks can be consumed by any process. setup :set_ersatz_global"},{"ref":"Ersatz.html#set_ersatz_private/1","title":"Ersatz.set_ersatz_private/1","type":"function","doc":"Sets the Ersatz to private mode, where mocks can be set and consumed by the same process unless other processes are explicitly allowed. setup :set_ersatz_private"},{"ref":"Ersatz.html#set_mock_implementation/3","title":"Ersatz.set_mock_implementation/3","type":"function","doc":"TODO: write doc for set_mock_implementation/3"},{"ref":"Ersatz.Matchers.Espec.html","title":"Ersatz.Matchers.Espec","type":"module","doc":"Custom matchers to ease the use of espec."},{"ref":"Ersatz.Matchers.Espec.html#have_been_called/1","title":"Ersatz.Matchers.Espec.have_been_called/1","type":"function","doc":"Tests if the mock function was called. It is possible to specify the number of calls to expect or just that it was called at least once. Default argument option is times: :at_least_once. Examples : ## Expect mock function to have been called at least once. expect(&amp;CalcMock.add/2) |&gt; to(have_been_called()) expect(&amp;CalcMock.add/2) |&gt; to(have_been_called(times: :at_least_once)) ## Expect mock function to have been called exactly 2 times. expect(&amp;CalcMock.add/2) |&gt; to(have_been_called(times: 2))"},{"ref":"Ersatz.Matchers.Espec.html#have_been_called_with/1","title":"Ersatz.Matchers.Espec.have_been_called_with/1","type":"function","doc":"Tests if the mock function was called with the right arguments. Two possibilities : Match on the totality of arguments with exactly: Assert that the mock function was called at least once with provided arguments at_least: Examples : ## Expect mock function to have been called with exactly twice with 1, 2 on first call and 4, 0 on second call. expect(&amp;CalcMock.add/2) |&gt; to(have_been_called_with(exactly: [{1, 2}, {4, 0}])) ## Expect mock function to have been called with at least once with 1, 2. expect(&amp;CalcMock.add/2) |&gt; to(have_been_called(at_least: {1, 2}))"},{"ref":"Ersatz.UnexpectedCallError.html","title":"Ersatz.UnexpectedCallError","type":"exception","doc":""},{"ref":"Ersatz.VerificationError.html","title":"Ersatz.VerificationError","type":"exception","doc":""}]